@@ -7,7 +7,7 @@
 ;;         Feng Shu <tumashu@163.com>
 ;; Maintainer: Feng Shu <tumashu@163.com>
 ;; URL: https://github.com/tumashu/pyim
-;; Version: 4.1.0
+;; Version: 5.2.3
 ;; Keywords: convenience, Chinese, pinyin, input-method
 ;; Package-Requires: ((emacs "25.1") (async "1.6") (xr "1.13"))
 
@@ -32,34 +32,27 @@
 
 ;; * 核心代码                                                           :code:
 ;; ** require + defcustom + defvar
-(require 'subr-x)
 (require 'cl-lib)
 (require 'help-mode)
+(require 'pyim-autoselector)
 (require 'pyim-common)
-(require 'pyim-scheme)
-(require 'pyim-page)
-(require 'pyim-process)
 (require 'pyim-cstring)
+(require 'pyim-dhashcache)
+;; NOTE: pyim 默认使用 dhashcache, 按理说不需要加载 dregcache, 但以前 pyim 支持
+;; 动态加载后端，由于简化代码的原因，这个功能取消了，require dregcache 是为了向
+;; 后兼容，未来也许会取消加载。
+(require 'pyim-dregcache)
+(require 'pyim-indicator)
+(require 'pyim-page)
+(require 'pyim-preview)
+(require 'pyim-process)
+(require 'pyim-scheme)
+(require 'subr-x)
 
 (defgroup pyim nil
   "Pyim is a Chinese input method support quanpin, shuangpin, wubi and cangjie."
   :group 'leim)
 
-(defcustom pyim-select-finish-hook nil
-  "Pyim 选词完成时运行的 hook."
-  :type 'hook)
-
-(defcustom pyim-convert-string-at-point-hook nil
-  "Hook of `pyim-convert-string-at-point'.
-
-这个 hook 运行时机：
-1. 获取并删除光标处 code 字符串之后。
-2. code 转换得到的中文字符串插入之前。
-
-Tip: 用户也可以利用 `pyim-outcome-trigger-function-default' 函数
-来构建适合自己的 hook 函数。"
-  :type 'hook)
-
 (defcustom pyim-select-word-by-number t
   "使用数字键来选择词条.
 
@@ -96,7 +89,7 @@ Tip: 用户也可以利用 `pyim-outcome-trigger-function-default' 函数
       (define-key map (kbd (car x))
                   (lambda ()
                     (interactive)
-                    (pyim-select-subword-by-number (cdr x)))))
+                    (pyim-select-char-in-word-by-number (cdr x)))))
     (define-key map " " #'pyim-select-word)
     (define-key map (kbd "C-SPC") #'pyim-select-word-simple)
     (define-key map [backspace] #'pyim-delete-backward-char)
@@ -128,6 +121,9 @@ Tip: 用户也可以利用 `pyim-outcome-trigger-function-default' 函数
     map)
   "Pyim 的 Keymap.")
 
+(cl-defmethod pyim-process-get-mode-map ()
+  pyim-mode-map)
+
 ;; ** pyim 输入法定义
 (defun pyim-input-method (key)
   "得到需要插入到 buffer 的字符串, 并将其插入到待输入 buffer.
@@ -140,10 +136,9 @@ Tip: 用户也可以利用 `pyim-outcome-trigger-function-default' 函数
           overriding-local-map)
       (list key)
     ;; (message "call with key: %S" key-or-string)
-    (pyim-process-init-ui)
     (with-silent-modifications
       (unwind-protect
-          (let ((input-string (pyim-input-method-1 key)))
+          (let ((input-string (pyim-process-input-method key)))
             ;; (message "input-string: %s" input-string)
             (when (and (stringp input-string)
                        (> (length input-string) 0))
@@ -152,74 +147,9 @@ Tip: 用户也可以利用 `pyim-outcome-trigger-function-default' 函数
                 (mapcar #'identity input-string))))
         (pyim-process-terminate)))))
 
-(defun pyim-input-method-1 (key)
-  "`pyim-input-method-1' 是 `pyim-input-method' 内部使用的函数。
-
-这个函数比较复杂，作许多低层工作，但它的一个重要流程是：
-
-1. 使用函数 `read-key-sequence' 得到 key-sequence
-2. 使用函数 `lookup-key' 查询 `pyim-mode-map' 中，与上述 key-sequence 对应
-   的命令。
-3. 如果查询得到的命令是 self-insert-command 时，调用这个函数。
-4. 这个函数最终会返回需要插入到 buffer 的字符串。
-
-这个部份的代码涉及许多 emacs 低层函数，相对复杂，不容易理解，有兴
-趣的朋友可以参考 elisp 手册相关章节:
-1. Invoking the Input Method
-2. Input Methods
-3. Miscellaneous Event Input Features
-4. Reading One Event"
-  ;; Check the possibility of translating KEY.
-  ;; If KEY is nil, we can anyway start translation.
-  (if (or (integerp key) (null key))
-      ;; OK, we can start translation.
-      (let* ((echo-keystrokes 0)
-             (help-char nil)
-             (overriding-terminal-local-map pyim-mode-map)
-             (input-method-function nil)
-             (input-method-use-echo-area nil)
-             (modified-p (buffer-modified-p))
-             last-command-event last-command this-command)
-
-        (setq pyim-process-translating t)
-        (pyim-process-cleanup-input-output)
-
-        (when key
-          (pyim-add-unread-command-events key))
-
-        (while pyim-process-translating
-          (set-buffer-modified-p modified-p)
-          (let* ((keyseq (read-key-sequence nil nil nil t))
-                 (cmd (lookup-key pyim-mode-map keyseq)))
-            ;; (message "key: %s, cmd:%s\nlcmd: %s, lcmdv: %s, tcmd: %s"
-            ;;          key cmd last-command last-command-event this-command)
-            (if (if key
-                    (commandp cmd)
-                  (pyim-process-self-insert-command-p cmd))
-                (progn
-                  ;; (message "keyseq: %s" keyseq)
-                  (setq last-command-event (aref keyseq (1- (length keyseq)))
-                        last-command this-command
-                        this-command cmd)
-                  (setq key t)
-                  (condition-case-unless-debug err
-                      (call-interactively cmd)
-                    (error (message "pyim 出现错误: %S , 开启 debug-on-error 后可以了解详细情况。" err)
-                           (beep))))
-              ;; KEYSEQ is not defined in the translation keymap.
-              ;; Let's return the event(s) to the caller.
-              (pyim-add-unread-command-events (this-single-command-raw-keys) t)
-              ;; (message "unread-command-events: %s" unread-command-events)
-              (pyim-process-terminate))))
-        ;; (message "return: %s" (pyim-process-get-outcome))
-        (pyim-process-get-outcome nil t t))
-    ;; Since KEY doesn't start any translation, just return it.
-    ;; But translate KEY if necessary.
-    (char-to-string key)))
-
 ;; ** Pyim 输入法注册
 ;;;###autoload
-(register-input-method "pyim" "UTF-8" #'pyim-activate pyim-title)
+(register-input-method "pyim" "UTF-8" #'pyim-activate pyim-title "")
 
 ;; ** PYim 输入法启动功能
 ;;;###autoload
@@ -231,7 +161,7 @@ pyim 是使用 `pyim-activate' 来启动输入法，这个命令主要做如下
 2. 创建汉字到拼音和拼音到汉字的 hash table。
 3. 创建词库缓存 dcache.
 4. 运行 hook： `pyim-load-hook'。
-5. 将 `pyim-kill-emacs-hook-function' 命令添加到 `kill-emacs-hook' , emacs 关闭
+5. 将 `pyim--kill-emacs-hook-function' 命令添加到 `kill-emacs-hook' , emacs 关闭
 之前将用户选择过的词生成的缓存和词频缓存保存到文件，供以后使用。
 6. 设定变量：
    1. `input-method-function'
@@ -253,11 +183,11 @@ pyim 使用函数 `pyim-activate' 启动输入法的时候，会将变量
   (pyim-process-init-dcaches)
 
   ;; 启动或者重启的时候，退出辅助输入法。
-  (setq pyim-assistant-scheme-enable nil)
+  (pyim-scheme-disable-assistant)
 
   (run-hooks 'pyim-load-hook)
   ;; Make sure personal or other dcache are saved to file before kill emacs.
-  (add-hook 'kill-emacs-hook #'pyim-kill-emacs-hook-function)
+  (add-hook 'kill-emacs-hook #'pyim--kill-emacs-hook-function)
 
   (setq deactivate-current-input-method-function #'pyim-deactivate)
   ;; If we are in minibuffer, turn off the current input method
@@ -268,9 +198,10 @@ pyim 使用函数 `pyim-activate' 启动输入法的时候，会将变量
   (setq-local input-method-function #'pyim-input-method)
   nil)
 
-(defun pyim-kill-emacs-hook-function ()
+(defun pyim--kill-emacs-hook-function ()
   "Pyim function which is used in `kill-emacs-hook'."
-  (pyim-process-save-dcaches t))
+  (pyim-process-save-dcaches t)
+  t)
 
 ;; ** 取消激活功能
 (defun pyim-deactivate ()
@@ -308,11 +239,16 @@ REFRESH-COMMON-DCACHE 已经废弃，不要再使用了。"
   (pyim-process-save-dcaches save-personal-dcache)
   (pyim-process-init-dcaches :force))
 
+;; ** 升级功能
+(defun pyim-upgrade ()
+  "升级 pyim 功能。"
+  (interactive)
+  (pyim-dcache-upgrade))
+
 ;; ** 键盘输入处理功能
 (defun pyim-self-insert-command ()
   "Pyim 默认的 self-insert-command."
   (interactive "*")
-  (setq pyim-candidates-last pyim-candidates)
   (cond
    ((pyim-process-input-chinese-p)
     (pyim-process-with-entered-buffer
@@ -321,24 +257,14 @@ REFRESH-COMMON-DCACHE 已经废弃，不要再使用了。"
       (insert (char-to-string last-command-event)))
     (pyim-process-run))
    ((pyim-process-get-candidates)
-    (pyim-process-outcome-handle 'candidate-and-last-char)
-    (pyim-process-terminate))
-   (t
-    (pyim-process-outcome-handle 'last-char)
-    (pyim-process-terminate))))
+    (pyim-process-select-word-and-last-char))
+   (t (pyim-process-select-last-char))))
 
-(cl-pushnew 'pyim-self-insert-command pyim-process-self-insert-commands)
+(pyim-process-register-self-insert-command 'pyim-self-insert-command)
 
 ;; ** 加词功能
-(defun pyim-create-word-at-point (&optional number silent)
-  "将光标前字符数为 NUMBER 的中文字符串添加到个人词库中，当
-SILENT 设置为 t 是，不显示提醒信息。"
-  (let ((string (pyim-cstring-at-point (or number 2)))
-        output)
-    (when string
-      (setq output (pyim-process-create-word string))
-      (unless silent
-        (message "将词条: %S 加入 personal 缓冲。" output)))))
+(defalias 'pyim-create-word-at-point
+  #'pyim-process-create-word-at-point)
 
 (defun pyim-create-2cchar-word-at-point ()
   "将光标前2个中文字符组成的字符串加入个人词库。"
@@ -366,7 +292,10 @@ SILENT 设置为 t 是，不显示提醒信息。"
         (if (not (string-match-p "^\\cc+\\'" string))
             (error "不是纯中文字符串")
           (setq output (pyim-process-create-word string))
-          (message "将词条: %S 插入 personal file。" output))))))
+          (message "将词条: %S 插入 personal file。" output)))
+      (deactivate-mark)
+      ;; NOTE: 这里必须返回 t, 因为这个函数的返回结果会被用来做为判断条件。
+      t)))
 
 ;; ** 导入词条功能
 (defun pyim-import-words-and-counts (file &optional merge-method silent)
@@ -421,13 +350,30 @@ MERGE-METHOD 是一个函数，这个函数需要两个数字参数，代表词
     ;; 有这一步骤，导入的词条就会被覆盖。
     (pyim-process-save-dcaches t)
     ;; 更新相关的 dcache
-    (pyim-process-update-personal-words)
+    (pyim-process-update t)
 
     (message "PYIM: 词条和词频信息导入完成！")))
 
 ;; ** 导出功能
-(defalias 'pyim-export-words-and-counts 'pyim-dcache-export-words-and-counts)
-(defalias 'pyim-export-personal-words 'pyim-dcache-export-personal-words)
+(defun pyim-export-words-and-counts (file &optional confirm ignore-counts)
+  "将个人词条以及词条对应的词频信息导出到文件 FILE.
+
+如果 FILE 为 nil, 提示用户指定导出文件位置, 如果 CONFIRM 为
+non-nil，文件存在时将会提示用户是否覆盖，默认为覆盖模式"
+  (interactive "F将词条和词频信息导出到文件: ")
+  (pyim-dcache-init-variables)
+  (pyim-dcache-export-words-and-counts file confirm ignore-counts)
+  (message "PYIM: 词条和词频信息导出完成。"))
+
+(defun pyim-export-personal-words (file &optional confirm)
+  "将用户的个人词条导出为 pyim 词库文件.
+
+如果 FILE 为 nil, 提示用户指定导出文件位置, 如果 CONFIRM 为 non-nil，
+文件存在时将会提示用户是否覆盖，默认为覆盖模式。"
+  (interactive "F将个人词条导出到文件：")
+  (pyim-dcache-init-variables)
+  (pyim-dcache-export-personal-words file confirm)
+  (message "PYIM: 个人词条导出完成。"))
 
 ;; ** 删词功能
 (defun pyim-delete-words-in-file (file)
@@ -455,18 +401,13 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
 (defun pyim-delete-last-word ()
   "从个人词库中删除最新创建的词条。"
   (interactive)
-  (when pyim-process-last-created-words
-    (pyim-process-delete-word (car pyim-process-last-created-words))
-    (message "pyim: 从个人词库中删除词条 “%s” !" (car pyim-process-last-created-words))))
+  (let ((word (pyim-process-last-created-word)))
+    (when word
+      (pyim-process-delete-word word)
+      (message "pyim: 从个人词库中删除词条 “%s” !" word))))
 
-(defun pyim-delete-word-at-point (&optional number silent)
-  "将光标前字符数为 NUMBER 的中文字符串从个人词库中删除
-当 SILENT 设置为 t 是，不显示提醒信息。"
-  (let ((string (pyim-cstring-at-point (or number 2))))
-    (when string
-      (pyim-process-delete-word string)
-      (unless silent
-        (message "词条: \"%s\" 已经从个人词库缓冲中删除。" string)))))
+(defalias 'pyim-delete-word-at-point
+  #'pyim-process-delete-word-at-point)
 
 (defun pyim-delete-word ()
   "从个人词库中删除词条。"
@@ -482,11 +423,9 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
         (deactivate-mark))))
    (t (let ((words (completing-read-multiple
                     "请选择需要删除的词条(可多选): "
-                    pyim-process-last-created-words)))
+                    (pyim-process-last-created-words))))
         (dolist (word words)
           (pyim-process-delete-word word)
-          (setq pyim-process-last-created-words
-                (remove word pyim-process-last-created-words))
           (message "将词条: %S 从 personal 缓冲中删除。" word))))))
 
 ;; ** 选词功能
@@ -498,130 +437,38 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
 然后再删除不需要的字，由于这个词条不是常用词条，所以
 不需要保存到个人词库。"
   (interactive)
-  (if (null (pyim-process-get-candidates))
-      (pyim-process-outcome-handle 'last-char)
-    (pyim-process-outcome-handle 'candidate))
-  (pyim-process-terminate))
+  (if (pyim-process-get-candidates)
+      (pyim-process-select-word-without-save)
+    (pyim-process-select-last-char)))
 
 (defun pyim-select-word ()
   "从选词框中选择当前词条，然后删除该词条对应拼音。"
   (interactive)
-  (pyim-process-create-code-criteria)
-  (if (null (pyim-process-get-candidates))  ; 如果没有选项，输入空格
-      (progn
-        (pyim-process-outcome-handle 'last-char)
-        (pyim-process-terminate))
-    (let* ((class (pyim-scheme-get-option (pyim-scheme-name) :class))
-           (func (intern (format "pyim-select-word:%S" class))))
-      (if (and class (functionp func))
-          (funcall func)
-        (call-interactively #'pyim-select-word:pinyin)))))
+  (if (pyim-process-get-candidates)
+      (pyim-process-select-word (pyim-scheme-current))
+    ;; 如果没有选项，输入空格
+    (pyim-process-select-last-char)))
 
-(defun pyim-select-word:pinyin ()
-  "从选词框中选择当前词条，然后删除该词条对应拼音。"
-  (interactive)
-  (pyim-process-outcome-handle 'candidate)
-  (let* ((imobj (car (pyim-process-get-imobjs)))
-         (length-selected-word
-          ;; 获取 *这一次* 选择词条的长度， 在“多次选择词条才能上屏”的情况下，
-          ;; 一定要和 output 的概念作区别。
-          ;; 比如： xiaolifeidao
-          ;; 第一次选择：小李， output = 小李
-          ;; 第二次选择：飞，   output = 小李飞
-          ;; 第三次选择：刀，   output = 小李飞刀
-          (- (length (pyim-process-get-outcome))
-             (length (pyim-process-get-outcome 1))))
-         ;; pyim-imobjs 包含 *pyim-entered-buffer* 里面光标前面的字符
-         ;; 串，通过与 selected-word 做比较，获取光标前未转换的字符串。
-         ;; to-be-translated.
-         (to-be-translated
-          (string-join (mapcar (lambda (w)
-                                 (concat (nth 2 w) (nth 3 w)))
-                               (nthcdr length-selected-word imobj)))))
-    ;; 大体来说，entered 字符串可以分解为三个部分：
-
-    ;; 1. 光标前字符串
-    ;;    1. 光标前已经转换的字符串
-    ;;    2. 光标前还没有转换的字符串。
-    ;; 2. 光标后字符串
-
-    ;; 下面对 entered 字符串的大体思路是：截取已经转换的字符串，把未转
-    ;; 换的字符串和光标后的字符串合并后下一轮递归的处理。
-
-    ;; 比如：entered 为 xiaolifeidao, 本次选择 “小李” 之后，需要将
-    ;; entered 截断，“小李” 这个词条长度为2, 就将 entered从头开始缩减
-    ;; 2 个 imelem 对应的字符，变成 feidao, 为下一次选择 “飞” 做准备。
-
-    ;; 注意事项： 这里有一个假设前提是： 一个 imelem 对应一个汉字，
-    ;; 在全拼输入法中，这个假设大多数情况是成立的，但在型码输入法
-    ;; 中，比如五笔输入法，就不成立，好在型码输入法一般不需要多次
-    ;; 选择。
-    (if (and (not (pyim-process-select-subword-p)) ;以词定字的时候，不连续选择，处理起来太复杂。
-             (or (< length-selected-word (length imobj)) ;是否有未转换的光标前字符串
-                 (> (length (pyim-process-get-entered 'point-after)) 0))) ;是否有光标后字符串
-        (progn
-          (pyim-process-with-entered-buffer
-            ;; 把光标前已转换的 entered 字符串, 从 entered字符串里面剪
-            ;; 掉，保留未转换的字符串和光标之后的字符串。
-            (delete-region (point-min) (point))
-            (insert to-be-translated)
-            ;; 为下一次选词作准备，一般情况下词库里面的词条不会超过20
-            ;; 个汉字，所以这里光标向前移动不超过20个 imelem. 从而让下
-            ;; 一轮处理时的“光标前字符串”比较长，这种方式可能比逐字选
-            ;; 择更加好用。
-            (goto-char (pyim-process-next-imelem-position 20 t 1)))
-          (pyim-process-run))
-      (pyim-process-create-word (pyim-process-get-outcome) t)
-      (pyim-process-terminate)
-      ;; pyim 使用这个 hook 来处理联想词。
-      (run-hooks 'pyim-select-finish-hook))))
-
-(defun pyim-select-word:xingma ()
-  "从选词框中选择当前词条，然后删除该词条对应编码。"
-  (interactive)
-  (pyim-process-outcome-handle 'candidate)
-  (if (pyim-process-with-entered-buffer
-        (and (> (point) 1)
-             (< (point) (point-max))))
-      (progn
-        (pyim-process-with-entered-buffer
-          ;; 把本次已经选择的词条对应的子 entered, 从 entered
-          ;; 字符串里面剪掉。
-          (delete-region (point-min) (point)))
-        (pyim-process-run))
-    ;; NOTE: 以词定字的时候，到底应不应该保存词条呢，需要进一步研究。
-    (pyim-process-create-word (pyim-process-get-outcome) t)
-    (pyim-process-terminate)
-    ;; pyim 使用这个 hook 来处理联想词。
-    (run-hooks 'pyim-select-finish-hook)))
-
-(defun pyim-select-word-by-number (&optional n)
+(defun pyim-select-word-by-number (&optional num)
   "使用数字编号来选择对应的词条。"
   (interactive)
-  (if (or pyim-select-word-by-number n)
-      (if (null (pyim-process-get-candidates))
-          (progn
-            (pyim-process-outcome-handle 'last-char)
-            (pyim-process-terminate))
-        (let ((index (if (numberp n)
-                         (- n 1)
-                       0))
-              (end (pyim-page-end)))
-          (when (= index -1) (setq index 9))
-          (if (> (+ index (pyim-page-start)) end)
-              (pyim-page-refresh)
-            (pyim-process-set-candidate-position
-             (+ (pyim-page-start) index))
-            (pyim-select-word))))
+  (if (or pyim-select-word-by-number num)
+      (pyim-select-word-by-number-1 num)
     ;; 有些输入法使用数字键编码，这种情况下，数字键就
     ;; 不能用来选词了。
     (call-interactively #'pyim-self-insert-command)))
 
-(defun pyim-select-subword-by-number (&optional n)
+(defun pyim-select-word-by-number-1 (num)
+  (if (and (pyim-process-get-candidates)
+           (pyim-page-plan-to-select-word num))
+      (pyim-process-select-word (pyim-scheme-current))
+    (pyim-process-select-last-char)))
+
+(defun pyim-select-char-in-word-by-number (&optional n)
   "以词定字功能。"
   (interactive)
-  (pyim-process-toggle-set-subword-info (or n 1))
-  (pyim-process-run t))
+  (pyim-process-plan-to-select-char-in-word (1- (or n 1)))
+  (pyim-process-select-word (pyim-scheme-current)))
 
 ;; ** 翻页和翻词功能
 (defalias 'pyim-previous-page #'pyim-page-previous-page)
@@ -633,22 +480,16 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
 (defun pyim-quit-clear ()
   "取消当前输入的命令."
   (interactive)
-  (pyim-process-outcome-handle "")
-  (pyim-process-terminate))
+  (pyim-process-select-nothing))
 
 ;; ** 字母上屏功能
 (defun pyim-quit-no-clear ()
   "字母上屏命令."
   (interactive)
-  (pyim-process-outcome-handle 'pyim-entered)
-  (pyim-process-terminate))
+  (pyim-process-select-entered))
 
 ;; ** 中英文输入模式切换
-(defun pyim-toggle-input-ascii ()
-  "pyim 切换中英文输入模式。同时调整标点符号样式。"
-  (interactive)
-  (setq pyim-process-input-ascii
-        (not pyim-process-input-ascii)))
+(defalias 'pyim-toggle-input-ascii #'pyim-process-toggle-input-ascii)
 
 ;; ** 主辅输入法切换功能
 (defun pyim-toggle-assistant-scheme ()
@@ -657,12 +498,9 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
 这个功能一般用于五笔等形码输入法，在忘记编码的时候临时用拼音输入
 中文。"
   (interactive)
-  (if (= (length (pyim-process-get-entered 'point-before)) 0)
-      (progn
-        (pyim-process-outcome-handle 'last-char)
-        (pyim-process-terminate))
-    (setq pyim-assistant-scheme-enable
-          (not pyim-assistant-scheme-enable))
+  (if (pyim-process-without-entered-p)
+      (pyim-process-select-last-char)
+    (pyim-scheme-toggle-assistant)
     (pyim-process-run)))
 
 ;; ** PYIM 输入操作命令
@@ -670,17 +508,15 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
   "光标前移"
   (interactive)
   (pyim-process-with-entered-buffer
-    (ignore-errors
-      (forward-char)))
-  (pyim-process-run t))
+    (forward-char))
+  (pyim-process-run))
 
 (defun pyim-backward-point ()
   "光标后移"
   (interactive)
   (pyim-process-with-entered-buffer
-    (ignore-errors
-      (backward-char)))
-  (pyim-process-run t))
+    (backward-char))
+  (pyim-process-run))
 
 (defun pyim-backward-imelem (&optional search-forward)
   "光标向后移动一个 imelem 对应的字符串
@@ -690,7 +526,7 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
   (let ((position (pyim-process-next-imelem-position 1 search-forward)))
     (pyim-process-with-entered-buffer
       (goto-char position))
-    (pyim-process-run t)))
+    (pyim-process-run)))
 
 (defun pyim-forward-imelem ()
   "光标向前移动一个 imelem 对应的字符串"
@@ -702,25 +538,23 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
   (interactive)
   (pyim-process-with-entered-buffer
     (end-of-line))
-  (pyim-process-run t))
+  (pyim-process-run))
 
 (defun pyim-beginning-of-line ()
   "光标移至行首"
   (interactive)
   (pyim-process-with-entered-buffer
     (beginning-of-line))
-  (pyim-process-run t))
+  (pyim-process-run))
 
 (defun pyim-delete-backward-char (&optional n)
   "向后删除1个字符"
   (interactive)
   (pyim-process-with-entered-buffer
-    (ignore-errors
-      (delete-char (- 0 (or n 1)))))
-  (if (> (length (pyim-process-get-entered 'point-before)) 0)
-      (pyim-process-run t)
-    (pyim-process-outcome-handle "")
-    (pyim-process-terminate)))
+    (delete-char (- 0 (or n 1))))
+  (if (pyim-process-without-entered-p)
+      (pyim-process-select-nothing)
+    (pyim-process-run)))
 
 (defun pyim-delete-forward-char ()
   "向前删除1个字符"
@@ -733,7 +567,7 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
   (let ((position (pyim-process-next-imelem-position 1 search-forward)))
     (pyim-process-with-entered-buffer
       (delete-region (point) position))
-    (pyim-process-run t)))
+    (pyim-process-run)))
 
 (defun pyim-delete-forward-imelem ()
   "向前删除一个 imelem 对应的字符串"
@@ -742,82 +576,19 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
 
 ;; ** 金手指功能
 ;;;###autoload
-(defun pyim-convert-string-at-point (&optional return-cregexp)
-  "将光标前的用户输入的字符串转换为中文.
-
-如果 RETURN-CREGEXP 为真, pyim 会把用户输入的字符串当作
-拼音，依照这个拼音来构建一个 regexp, 用户可以用这个 regexp
-搜索拼音对应的汉字。"
+(defun pyim-convert-string-at-point (&optional _)
+  "将光标前的用户输入的字符串转换为中文."
   (interactive "P")
+  (pyim--activate-pyim)
+  (or (pyim-create-word-from-selection)
+      (pyim-process-trigger-feature-run-p)
+      (pyim-process-feed-entered-at-point-into-pyim)
+      (message "PYIM: `pyim-convert-string-at-point' did nothing.")))
+
+(defun pyim--activate-pyim ()
+  "如果当前输入法设置为 pyim, 就激活它。"
   (unless (equal input-method-function 'pyim-input-method)
-    (activate-input-method 'pyim))
-  (if return-cregexp
-      (pyim-cregexp-convert-at-point t)
-    (let* ((case-fold-search nil)
-           (scheme-name (pyim-scheme-name))
-           (first-chars (pyim-scheme-get-option scheme-name :first-chars))
-           (rest-chars (pyim-scheme-get-option scheme-name :rest-chars))
-           (string (if mark-active
-                       (buffer-substring-no-properties
-                        (region-beginning) (region-end))
-                     (buffer-substring (point) (line-beginning-position))))
-           (str-before-1 (pyim-char-before-to-string 0))
-           (str-before-2 (pyim-char-before-to-string 1))
-           (str-before-3 (pyim-char-before-to-string 2))
-           code length)
-      (cond
-       ;; 如果用户已经选择词条，就将此词条添加到个人词库。
-       ((region-active-p)
-        (pyim-create-word-from-selection)
-        (deactivate-mark))
-       ;; 删除用户自定义词条。比如：在一个中文字符串后输入 2-，运行此命令可以将
-       ;; 光标前两个中文字符组成的字符串，从个人词库删除。
-       ((and (eq (char-before) ?-)
-             (pyim-string-match-p "[0-9]" str-before-2)
-             (pyim-string-match-p "\\cc" str-before-3))
-        (delete-char -2)
-        (pyim-delete-word-at-point
-         (string-to-number str-before-2)))
-       ;; 输入"-"然后运行此命令，可以快速删除最近一次创建的词条。
-       ((and (eq (char-before) ?-)
-             (pyim-string-match-p "\\cc" str-before-2))
-        (delete-char -1)
-        (pyim-delete-last-word))
-       ;; 快速保存用户自定义词条。比如：在一个中文字符串后输入 2，运行此命令可以
-       ;; 将光标前两个中文字符组成的字符串，保存到个人词库。
-       ((and (member (char-before) (number-sequence ?2 ?9))
-             (pyim-string-match-p "\\cc" str-before-2))
-        (delete-char -1)
-        (pyim-create-word-at-point
-         (string-to-number str-before-1)))
-       ;; 金手指功能
-       ((string-match
-         ;; 创建一个 regexp, 用于提取出光标处一个适合
-         ;; 转换的字符串。
-         (format "[%s]+ *$"
-                 (cl-delete-duplicates
-                  (concat first-chars rest-chars "'-")))
-         string)
-        (setq code
-              ;; 一些编程语言使用单引号 ' 做为字符串的标记，这里需要特殊处理。
-              (replace-regexp-in-string
-               "^[-']" ""
-               (match-string 0 string)))
-        (setq length (length code))
-        (setq code (replace-regexp-in-string " +" "" code))
-        (when mark-active
-          (delete-region
-           (region-beginning) (region-end)))
-        (when (and (not mark-active) (> length 0))
-          (delete-char (- 0 length)))
-        (run-hooks 'pyim-convert-string-at-point-hook)
-        (when (> length 0)
-          (pyim-add-unread-command-events code)
-          (setq pyim-process-force-input-chinese t)))
-       ;; 当光标前的一个字符是标点符号时，半角/全角切换。
-       ((pyim-string-match-p "[[:punct:]：－]" (pyim-char-before-to-string 0))
-        (call-interactively 'pyim-punctuation-translate-at-point))
-       (t (message "Pyim: pyim-convert-string-at-point did nothing."))))))
+    (activate-input-method 'pyim)))
 
 ;; ** 编码反查功能
 (defun pyim-search-word-code ()
@@ -828,30 +599,17 @@ FILE 的格式与 `pyim-dcache-export' 生成的文件格式相同，
            codes)
       (if (not (string-match-p "^\\cc+\\'" string))
           (error "PYIM: 不是纯中文字符串。")
-        (setq codes (pyim-cstring-to-codes string pyim-default-scheme))
+        (setq codes (pyim-cstring-to-codes string (pyim-scheme-current)))
         (if codes
             (message "PYIM (%S): %S -> %S" pyim-default-scheme string codes)
           (message "PYIM: 没有找到 %S 对应的编码。" string)))
       (deactivate-mark))))
 
-;; ** pyim 中文字符串工具
-(require 'pyim-cstring)
-(defalias 'pyim-forward-word 'pyim-cstring-forward-word)
-(defalias 'pyim-backward-word 'pyim-cstring-backward-word)
-;; PYIM 重构以前使用的一些函数名称，alias 一下，便于兼容。
-(defalias 'pyim-hanzi2pinyin-simple 'pyim-cstring-to-pinyin-simple)
-(defalias 'pyim-hanzi2pinyin 'pyim-cstring-to-pinyin)
-(defalias 'pyim-hanzi2xingma 'pyim-cstring-to-xingma)
-(defalias 'pyim-cwords-at-point 'pyim-cstring-words-at-point)
-
-;; ** pyim 中文 regexp 工具
-(require 'pyim-cregexp)
-
 ;; ** pyim 探针
 (require 'pyim-probe)
 
-;; ** pyim hacks
-(require 'pyim-hacks)
+;; ** pyim 云输入法
+(require 'pyim-cloudim)
 
 ;; * Footer
 (provide 'pyim)
